xdel(winsid());
clear demo_compute;

f = createWindow();
f.axes_size = [800,600];
f.layout = "border";
f.resize = "off";
f.figure_name = "TestPythonCoSimulation FMU";
f.background = -2;

//params
frL = uicontrol(f, ...
    "style", "frame", ...
    "constraints", createConstraints("border", "left", [200 0]), ...
    "relief", "solid", ...
    "tag", "left");

//Title simu
uicontrol(frL, ...
    "style", "text", ...
    "string", "Simulation parameters", ...
    "fontweight", "bold", ...
    "fontsize", 16, ...
    "horizontalalignment", "center", ...
    "position", [1 550 198 40]);
    

//start time
uicontrol(frL, ...
    "style", "text", ...
    "string", "Start time", ...
    "position", [25 520 50 25]);
    
uicontrol(frL, ...
    "style", "edit", ...
    "string", "0", ...
    "tag", "start", ...
    "horizontalalignment", "center", ...
    "position", [80 520 100 25]);

//end time
uicontrol(frL, ...
    "style", "text", ...
    "string", "End time", ...
    "position", [25 480 50 25]);
    
uicontrol(frL, ...
    "style", "edit", ...
    "string", "1", ...
    "tag", "stop", ...
    "horizontalalignment", "center", ...
    "position", [80 480 100 25]);

//step time
uicontrol(frL, ...
    "style", "text", ...
    "string", "Step size", ...
    "position", [25 440 50 25]);
    
uicontrol(frL, ...
    "style", "edit", ...
    "string", "0.01", ...
    "tag", "step", ...
    "horizontalalignment", "center", ...
    "position", [80 440 100 25]);

//Title FMU
uicontrol(frL, ...
    "style", "text", ...
    "string", "FMU parameters", ...
    "fontweight", "bold", ...
    "fontsize", 16, ...
    "horizontalalignment", "center", ...
    "position", [1 350 198 50]);
    
//FMU param
uicontrol(frL, ...
    "style", "text", ...
    "string", "m", ...
    "position", [25 320 50 25]);
    
uicontrol(frL, ...
    "style", "edit", ...
    "string", "1", ...
    "tag", "m", ...
    "horizontalalignment", "center", ...
    "position", [80 320 100 25]);

uicontrol(frL, ...
    "style", "text", ...
    "string", "k", ...
    "position", [25 280 50 25]);
    
uicontrol(frL, ...
    "style", "edit", ...
    "string", "1000", ...
    "tag", "k", ...
    "horizontalalignment", "center", ...
    "position", [80 280 100 25]);

uicontrol(frL, ...
    "style", "pushbutton", ...
    "string", "Simulate", ...
    "fontweight", "bold", ...
    "fontsize", 14, ...
    "position", [25 200 150 30], ...
    "callback", "demo_compute");


//plot
frC = uicontrol(f, ...
    "style", "frame", ...
    "tag", "center");


function demo_compute()

    
    path = fmigetPath();
    f = fullfile(path, "demos", "TestPythonCoSimulation.fmu");
    model = importFMU(f);
    inst = fmiInstantiateSlave(model, 5, %f);
    
    tStart = strtod(get("start", "string"));
    tStop = strtod(get("stop", "string"));
    ts = strtod(get("step", "string"))

    realVar = model.modelVariables.Real;

    idxX = find(realVar.name == "mass1.x");
    refX = realVar.valueReference(idxX);

    idxV = find(realVar.name == "mass1.v");
    refV = realVar.valueReference(idxV);

    m = strtod(get("m", "string"));
    idxM = find(realVar.name == "mass1.m");
    refM = realVar.valueReference(idxM);
    fmiSetReal(inst, refM, m);

    k = strtod(get("k", "string"));
    idxK = find(realVar.name == "springDamper1.k");
    refK = realVar.valueReference(idxK);
    fmiSetReal(inst, refK, k);

    time=tStart:ts:tStop;
    xvalues = zeros(size(time));
    vvalues = zeros(size(time));

    idx=1;

    status = fmiInitializeSlave(inst, tStart, tStop);
    tic();
    while idx<=size(time,2)
        tc=time(idx);
        
        //mass1.x
        xvalues(idx) = fmiGetReal(inst, refX);
        //mass1.v
        vvalues(idx) = fmiGetReal(inst, refV);
       
        fmiDoStep(inst, tc, ts, %t);
       
        idx = idx + 1;
    end
    t = toc();
    fmiTerminateSlave(inst);
    fmiFreeSlaveInstance(inst);

    axe = get("axe");
    delete(axe);
    axe = newaxes(get("center"));
    axe.tag = "axe";

    plot(time, xvalues, time, vvalues);
endfunction  
